from textual.app import App, ComposeResult
from textual.widgets import Header, Footer
from textual import on
from textual.screen import ModalScreen
from mastui.login import LoginScreen
from mastui.post import PostScreen
from mastui.reply import ReplyScreen
from mastui.splash import SplashScreen
from mastui.mastodon_api import get_api
from mastui.timeline import Timelines, Timeline
from mastui.widgets import Post, LikePost, BoostPost
from mastui.thread import ThreadScreen
from mastui.profile import ProfileScreen
from mastui.config_screen import ConfigScreen
from mastui.help_screen import HelpScreen
from mastui.logging_config import setup_logging
import logging
import argparse
import os
from mastui.config import config
from mastui.messages import (
    PostStatusUpdate,
    ActionFailed,
    TimelineData,
    FocusNextTimeline,
    FocusPreviousTimeline,
    ViewProfile,
)

# Set up logging
log = logging.getLogger(__name__)


# Get the absolute path to the CSS file
css_path = os.path.join(os.path.dirname(__file__), "app.css")


class Mastui(App):
    """A Textual app to interact with Mastodon."""

    BINDINGS = [
        ("d", "toggle_dark", "Toggle dark mode"),
        ("r", "refresh_timelines", "Refresh timelines"),
        ("c", "compose_post", "Compose post"),
        ("p", "view_profile", "View profile"),
        ("a", "reply_to_post", "Reply to post"),
        ("o", "open_options", "Options"),
        ("?", "show_help", "Help"),
        ("q", "quit", "Quit"),
        ("l", "like_post", "Like post"),
        ("b", "boost_post", "Boost post"),
        ("up", "scroll_up", "Scroll up"),
        ("down", "scroll_down", "Scroll down"),
    ]
    CSS_PATH = css_path
    initial_data = None
    max_characters = 500 # Default value
    log_file_path: str | None = None

    def compose(self) -> ComposeResult:
        """Create child widgets for the app."""
        yield Header()
        yield Footer()

    def on_mount(self) -> None:
        """Called when the app is mounted."""
        self.theme = config.theme
        self.theme_changed_signal.subscribe(self, self.on_theme_changed)
        self.push_screen(SplashScreen())
        self.api = get_api()
        if self.api:
            self.run_worker(self.fetch_instance_info, thread=True, exclusive=True)
            self.set_timer(2, self.show_timelines)
        else:
            self.call_later(self.show_login_screen)

    def fetch_instance_info(self):
        """Fetches instance information from the API."""
        try:
            instance = self.api.instance()
            self.max_characters = instance['configuration']['statuses']['max_characters']
        except Exception as e:
            log.error(f"Error fetching instance info: {e}", exc_info=True)
            self.notify("Could not fetch instance information.", severity="error")

    def on_theme_changed(self, event) -> None:
        """Called when the app's theme is changed."""
        new_theme = event.name
        config.theme = new_theme
        if "light" in new_theme:
            config.preferred_light_theme = new_theme
        else:
            config.preferred_dark_theme = new_theme
        config.save_config()

    def show_login_screen(self):
        if isinstance(self.screen, SplashScreen):
            self.pop_screen()
        self.push_screen(LoginScreen(), self.on_login)

    def on_login(self, api) -> None:
        """Called when the login screen is dismissed."""
        log.info("Login successful.")
        self.api = api
        self.show_timelines()

    def show_timelines(self):
        if isinstance(self.screen, SplashScreen):
            self.pop_screen()
        log.info("Showing timelines...")
        self.mount(Timelines())
        self.call_later(self.focus_home_timeline)

    def focus_home_timeline(self):
        """Focuses the home timeline."""
        try:
            self.query_one("#home", Timeline).focus()
        except Exception as e:
            log.error(f"Could not focus home timeline: {e}", exc_info=True)

    def action_toggle_dark(self) -> None:
        """An action to toggle dark mode."""
        if "light" in self.theme:
            self.theme = config.preferred_dark_theme
        else:
            self.theme = config.preferred_light_theme

    def action_open_options(self) -> None:
        """An action to open the options screen."""
        if isinstance(self.screen, ModalScreen):
            return
        self.push_screen(ConfigScreen(), self.on_config_screen_dismiss)

    def action_show_help(self) -> None:
        """An action to show the help screen."""
        if isinstance(self.screen, ModalScreen):
            return
        self.push_screen(HelpScreen())

    def on_config_screen_dismiss(self, result: bool) -> None:
        """Called when the config screen is dismissed."""
        if result:
            for timeline in self.query(Timeline):
                timeline.update_auto_refresh_timer()

    def action_refresh_timelines(self) -> None:
        """An action to refresh the timelines."""
        log.info("Refreshing all timelines...")
        for timeline in self.query(Timeline):
            timeline.refresh_posts()

    def action_compose_post(self) -> None:
        """An action to compose a new post."""
        if isinstance(self.screen, ModalScreen):
            return
        self.push_screen(PostScreen(max_characters=self.max_characters), self.on_post_screen_dismiss)

    def action_reply_to_post(self) -> None:
        focused = self.query("Timeline:focus")
        if focused:
            focused.first().reply_to_post()

    def on_post_screen_dismiss(self, result: dict) -> None:
        """Called when the post screen is dismissed."""
        if result:
            try:
                log.info("Sending post...")
                self.api.status_post(
                    status=result["content"],
                    spoiler_text=result["spoiler_text"],
                    language=result["language"],
                )
                log.info("Post sent successfully.")
                self.notify("Post sent successfully!", severity="information")
                self.action_refresh_timelines()
            except Exception as e:
                log.error(f"Error sending post: {e}", exc_info=True)
                self.notify(f"Error sending post: {e}", severity="error")

    def on_reply_screen_dismiss(self, result: dict) -> None:
        """Called when the reply screen is dismissed."""
        if result:
            try:
                log.info(f"Sending reply to post {result['in_reply_to_id']}...")
                self.api.status_post(
                    status=result["content"],
                    spoiler_text=result["spoiler_text"],
                    language=result["language"],
                    in_reply_to_id=result["in_reply_to_id"],
                )
                log.info("Reply sent successfully.")
                self.notify("Reply sent successfully!", severity="information")
                self.action_refresh_timelines()
            except Exception as e:
                log.error(f"Error sending reply: {e}", exc_info=True)
                self.notify(f"Error sending reply: {e}", severity="error")

    @on(LikePost)
    def handle_like_post(self, message: LikePost):
        self.run_worker(lambda: self.do_like_post(message.post_id), exclusive=True, thread=True)

    def do_like_post(self, post_id: str):
        try:
            # Find the post to determine its current liked status
            post_widget = self.query_one(f"#post-{post_id}", Post)
            status_to_action = post_widget.post.get("reblog") or post_widget.post
            
            if status_to_action.get("favourited"):
                post_data = self.api.status_unfavourite(post_id)
            else:
                post_data = self.api.status_favourite(post_id)
            self.post_message(PostStatusUpdate(post_data))
        except Exception as e:
            log.error(f"Error liking/unliking post {post_id}: {e}", exc_info=True)
            self.post_message(ActionFailed(post_id))

    @on(BoostPost)
    def handle_boost_post(self, message: BoostPost):
        self.run_worker(lambda: self.do_boost_post(message.post_id), exclusive=True, thread=True)

    def do_boost_post(self, post_id: str):
        try:
            post_data = self.api.status_reblog(post_id)
            self.post_message(PostStatusUpdate(post_data))
        except Exception as e:
            log.error(f"Error boosting post {post_id}: {e}", exc_info=True)
            self.post_message(ActionFailed(post_id))

    def on_post_status_update(self, message: PostStatusUpdate) -> None:
        updated_post_data = message.post_data
        target_post = updated_post_data.get("reblog") or updated_post_data
        target_id = target_post["id"]

        for container in [self.screen, *self.query(Timelines)]:
            for post_widget in container.query(Post):
                original_status = post_widget.post.get("reblog") or post_widget.post
                if original_status["id"] == target_id:
                    post_widget.update_from_post(updated_post_data)

    def on_action_failed(self, message: ActionFailed) -> None:
        for container in [self.screen, *self.query(Timelines)]:
            for post_widget in container.query(Post):
                original_status = post_widget.post.get("reblog") or post_widget.post
                if original_status["id"] == message.post_id:
                    post_widget.hide_spinner()

    @on(FocusNextTimeline)
    def on_focus_next_timeline(self, message: FocusNextTimeline) -> None:
        timelines = self.query(Timeline)
        for i, timeline in enumerate(timelines):
            if timeline.has_focus:
                timelines[(i + 1) % len(timelines)].focus()
                return

    @on(FocusPreviousTimeline)
    def on_focus_previous_timeline(self, message: FocusPreviousTimeline) -> None:
        timelines = self.query(Timeline)
        for i, timeline in enumerate(timelines):
            if timeline.has_focus:
                timelines[(i - 1) % len(timelines)].focus()
                return

    @on(ViewProfile)
    def on_view_profile(self, message: ViewProfile) -> None:
        if isinstance(self.screen, ModalScreen):
            return
        self.push_screen(ProfileScreen(message.account_id, self.api))

    def action_like_post(self) -> None:
        focused = self.query("Timeline:focus")
        if focused:
            focused.first().like_post()

    def action_boost_post(self) -> None:
        focused = self.query("Timeline:focus")
        if focused:
            focused.first().boost_post()

    def action_scroll_up(self) -> None:
        focused = self.query("Timeline:focus")
        if focused:
            focused.first().scroll_up()

    def action_scroll_down(self) -> None:
        focused = self.query("Timeline:focus")
        if focused:
            focused.first().scroll_down()


def main():
    parser = argparse.ArgumentParser(description="A Textual app to interact with Mastodon.")
    parser.add_argument("--no-ssl-verify", action="store_false", dest="ssl_verify", help="Disable SSL verification.")
    parser.add_argument("--debug", action="store_true", help="Enable debug logging.")
    args = parser.parse_args()
    
    log_file_path = setup_logging(debug=args.debug)
    
    config.ssl_verify = args.ssl_verify
    app = Mastui()
    app.log_file_path = log_file_path
    
    try:
        app.run()
    finally:
        if app.log_file_path:
            print(f"Log file written to: {app.log_file_path}")


if __name__ == "__main__":
    main()



import logging
import os
from pathlib import Path

def setup_logging(debug=False):
    """Set up logging to a file if debug mode is enabled."""
    if not debug:
        logging.basicConfig(level=logging.WARNING)
        return None

    config_dir = Path.home() / ".config" / "mastui"
    config_dir.mkdir(parents=True, exist_ok=True)
    log_file = config_dir / "mastui.log"

    logging.basicConfig(
        level=logging.DEBUG,
        format="%(asctime)s - %(levelname)s - %(message)s",
        filename=log_file,
        filemode="w",  # Overwrite the log file on each run
    )
    return str(log_file)



from textual.screen import ModalScreen
from textual.widgets import Static
from textual.containers import VerticalScroll, Container
from textual.events import Key
from mastui.widgets import Post, Notification, LikePost, BoostPost
from mastui.reply import ReplyScreen
import logging

log = logging.getLogger(__name__)

class ThreadScreen(ModalScreen):
    """A modal screen to display a post thread."""

    DEFAULT_CSS = """
    ThreadScreen {
        align: center middle;
    }
    """

    BINDINGS = [
        ("r", "refresh_thread", "Refresh thread"),
        ("escape", "dismiss", "Close thread"),
        ("l", "like_post", "Like post"),
        ("b", "boost_post", "Boost post"),
        ("a", "reply_to_post", "Reply to post"),
        ("up", "scroll_up", "Scroll up"),
        ("down", "scroll_down", "Scroll down"),
    ]

    def __init__(self, post_id: str, api, **kwargs) -> None:
        super().__init__(**kwargs)
        self.post_id = post_id
        self.api = api
        self.selected_item = None

    def compose(self):
        with Container(id="thread-dialog"):
            yield VerticalScroll(
                Static("Loading thread...", classes="status-message"),
                id="thread-container"
            )

    def on_mount(self):
        self.run_worker(self.load_thread, thread=True)

    def action_refresh_thread(self):
        """Refresh the thread."""
        self.run_worker(self.load_thread, exclusive=True, thread=True)

    def load_thread(self):
        """Load the thread context."""
        try:
            context = self.api.status_context(self.post_id)
            main_post_data = self.api.status(self.post_id)
            self.app.call_from_thread(self.render_thread, context, main_post_data)
        except Exception as e:
            log.error(f"Error loading thread: {e}", exc_info=True)
            self.app.notify(f"Error loading thread: {e}", severity="error")
            self.dismiss()

    def render_thread(self, context, main_post_data):
        """Render the thread."""
        container = self.query_one("#thread-container")
        container.query("*").remove()

        ancestors = context.get("ancestors", [])
        descendants = context.get("descendants", [])
        
        for post in ancestors:
            container.mount(Post(post))

        main_post = Post(main_post_data)
        main_post.add_class("main-post")
        container.mount(main_post)

        for post in descendants:
            reply_post = Post(post)
            reply_post.add_class("reply-post")
            container.mount(reply_post)
            
        self.select_first_item()

    def select_first_item(self):
        if self.selected_item:
            self.selected_item.remove_class("selected")
        try:
            self.selected_item = self.query(Post).first()
            self.selected_item.add_class("selected")
        except Exception as e:
            log.error(f"Could not select first item in thread: {e}", exc_info=True)
            self.selected_item = None

    def on_key(self, event: Key) -> None:
        if event.key == "down":
            self.scroll_down()
            event.stop()
        elif event.key == "up":
            self.scroll_up()
            event.stop()
        elif event.key == "l":
            self.action_like_post()
            event.stop()
        elif event.key == "b":
            self.action_boost_post()
            event.stop()
        elif event.key == "a":
            self.action_reply_to_post()
            event.stop()

    def scroll_up(self):
        items = self.query("Post")
        if self.selected_item and items:
            try:
                idx = items.nodes.index(self.selected_item)
                if idx > 0:
                    self.selected_item.remove_class("selected")
                    self.selected_item = items[idx - 1]
                    self.selected_item.add_class("selected")
                    self.selected_item.scroll_visible()
            except ValueError as e:
                log.error(f"Could not scroll up in thread: {e}", exc_info=True)
                self.select_first_item()

    def scroll_down(self):
        items = self.query("Post")
        if self.selected_item and items:
            try:
                idx = items.nodes.index(self.selected_item)
                if idx < len(items) - 1:
                    self.selected_item.remove_class("selected")
                    self.selected_item = items[idx + 1]
                    self.selected_item.add_class("selected")
                    self.selected_item.scroll_visible()
            except ValueError as e:
                log.error(f"Could not scroll down in thread: {e}", exc_info=True)
                self.select_first_item()

    def action_like_post(self):
        if isinstance(self.selected_item, Post):
            status_to_action = self.selected_item.post.get("reblog") or self.selected_item.post
            if not status_to_action:
                self.app.notify("Cannot like a post that has been deleted.", severity="error")
                return
            self.selected_item.show_spinner()
            self.post_message(LikePost(status_to_action["id"]))

    def action_boost_post(self):
        if isinstance(self.selected_item, Post):
            status_to_action = self.selected_item.post.get("reblog") or self.selected_item.post
            if not status_to_action:
                self.app.notify("Cannot boost a post that has been deleted.", severity="error")
                return
            self.selected_item.show_spinner()
            self.post_message(BoostPost(status_to_action["id"]))

    def action_reply_to_post(self):
        if isinstance(self.selected_item, Post):
            post_to_reply_to = self.selected_item.post.get("reblog") or self.selected_item.post
            if post_to_reply_to:
                self.app.push_screen(ReplyScreen(post_to_reply_to), self.app.on_reply_screen_dismiss)
            else:
                self.app.notify("This item cannot be replied to.", severity="error")


from textual.screen import ModalScreen
from textual.widgets import DataTable, Static
from textual.containers import Vertical

class HelpScreen(ModalScreen):
    """A modal screen to display help information."""

    BINDINGS = [
        ("escape", "app.pop_screen", "Close Help"),
    ]

    DEFAULT_CSS = """
    HelpScreen {
        align: center middle;
    }
    #help-dialog {
        width: 80;
        height: auto;
        max-height: 80%;
        border: thick $primary 80%;
        background: $surface;
        padding: 1;
    }
    DataTable {
        margin-top: 1;
    }
    """

    def compose(self):
        with Vertical(id="help-dialog"):
            yield Static("Key Bindings", classes="title")
            table = DataTable()
            table.add_columns("Key", "Action", "Context")
            
            # General App Bindings
            table.add_row("[bold]Global[/bold]")
            table.add_row("d", "Toggle dark/light mode", "Anywhere")
            table.add_row("r", "Refresh all timelines", "Anywhere")
            table.add_row("c", "Compose new post", "Anywhere")
            table.add_row("o", "Open options screen", "Anywhere")
            table.add_row("?", "Show this help screen", "Anywhere")
            table.add_row("q", "Quit the application", "Anywhere")

            # Timeline Bindings
            table.add_row("")
            table.add_row("[bold]Timeline / Thread View[/bold]")
            table.add_row("up/down", "Navigate posts", "Timeline/Thread")
            table.add_row("left/right", "Switch between timelines", "Timeline")
            table.add_row("enter", "View post thread", "Timeline")
            table.add_row("a", "Reply to selected post", "Timeline/Thread")
            table.add_row("l", "Like selected post", "Timeline/Thread")
            table.add_row("b", "Boost selected post", "Timeline/Thread")
            table.add_row("p", "View profile of post author", "Timeline/Thread")

            # Profile Screen Bindings
            table.add_row("")
            table.add_row("[bold]Profile View[/bold]")
            table.add_row("f", "Follow/Unfollow user", "Profile")

            # Modal Screen Bindings
            table.add_row("")
            table.add_row("[bold]Modals[/bold]")
            table.add_row("escape", "Close the current modal/dialog", "Any Modal")

            yield table



import html2text
import pprint
from datetime import datetime, timezone
from dateutil.parser import parse

LANGUAGE_OPTIONS = [
    ("Chinese", "zh"),
    ("Danish", "da"),
    ("English", "en"),
    ("French", "fr"),
    ("German", "de"),
    ("Japanese", "ja"),
    ("Korean", "ko"),
    ("Spanish", "es"),
]


def to_markdown(html):
    """Converts HTML to markdown."""
    h = html2text.HTML2Text()
    h.ignore_links = False
    return h.handle(html)

def get_full_content_md(status):
    """Gets the full markdown content for a status, including media."""
    if not status:
        return ""
        
    html_content = status.get('content') or status.get('note') or ''
    content_md = to_markdown(html_content)

    if status.get("media_attachments"):
        media_infos = []
        for media in status["media_attachments"]:
            media_type = media.get("type", "media").capitalize()
            description = media.get("description")

            if description:
                media_infos.append(f"[{media_type} showing: {description}]")
            else:
                media_infos.append(f"[{media_type} attached]")

        if content_md.strip():
            content_md += "\n\n" + "\n".join(media_infos)
        else:
            content_md = "\n".join(media_infos)

    if not content_md.strip():
        pretty_status = pprint.pformat(status)
        return f"**Empty Post Detected! Raw data:**\n```json\n{pretty_status}\n```"

    return content_md

def format_datetime(dt_obj):
    """Formats a datetime string or object into YYYY-MM-DD HH:MM."""
    if isinstance(dt_obj, str):
        dt = parse(dt_obj)
    else:
        dt = dt_obj

    # Convert to local timezone for display
    local_dt = dt.astimezone()
    
    return local_dt.strftime("%Y-%m-%d %H:%M")



import os
from dotenv import load_dotenv
from pathlib import Path

class Config:
    def __init__(self):
        self.config_dir = Path.home() / ".config" / "mastui"
        self.config_dir.mkdir(parents=True, exist_ok=True)
        self.env_file = self.config_dir / ".env"
        
        if self.env_file.exists():
            load_dotenv(self.env_file)

        self.mastodon_host = os.getenv("MASTODON_HOST")
        self.mastodon_client_id = os.getenv("MASTODON_CLIENT_ID")
        self.mastodon_client_secret = os.getenv("MASTODON_CLIENT_SECRET")
        self.mastodon_access_token = os.getenv("MASTODON_ACCESS_TOKEN")
        self.ssl_verify = True
        
        self.theme = os.getenv("THEME", "textual-dark")
        self.preferred_dark_theme = os.getenv("PREFERRED_DARK_THEME", "textual-dark")
        self.preferred_light_theme = os.getenv("PREFERRED_LIGHT_THEME", "textual-light")

        # Auto-refresh settings
        self.home_auto_refresh = os.getenv("HOME_AUTO_REFRESH", "on") == "on"
        self.home_auto_refresh_interval = int(os.getenv("HOME_AUTO_REFRESH_INTERVAL", "2"))
        self.notifications_auto_refresh = os.getenv("NOTIFICATIONS_AUTO_REFRESH", "on") == "on"
        self.notifications_auto_refresh_interval = int(os.getenv("NOTIFICATIONS_AUTO_REFRESH_INTERVAL", "10"))
        self.federated_auto_refresh = os.getenv("FEDERATED_AUTO_REFRESH", "on") == "on"
        self.federated_auto_refresh_interval = int(os.getenv("FEDERATED_AUTO_REFRESH_INTERVAL", "2"))

        # Image settings
        self.image_support = os.getenv("IMAGE_SUPPORT", "off") == "on"
        self.image_renderer = os.getenv("IMAGE_RENDERER", "ansi")


    def save_config(self):
        with open(self.env_file, "w") as f:
            if self.mastodon_host:
                f.write(f"MASTODON_HOST={self.mastodon_host}\n")
            if self.mastodon_client_id:
                f.write(f"MASTODON_CLIENT_ID={self.mastodon_client_id}\n")
            if self.mastodon_client_secret:
                f.write(f"MASTODON_CLIENT_SECRET={self.mastodon_client_secret}\n")
            if self.mastodon_access_token:
                f.write(f"MASTODON_ACCESS_TOKEN={self.mastodon_access_token}\n")
            if self.theme:
                f.write(f"THEME={self.theme}\n")
            if self.preferred_dark_theme:
                f.write(f"PREFERRED_DARK_THEME={self.preferred_dark_theme}\n")
            if self.preferred_light_theme:
                f.write(f"PREFERRED_LIGHT_THEME={self.preferred_light_theme}\n")
            
            f.write(f"HOME_AUTO_REFRESH={'on' if self.home_auto_refresh else 'off'}\n")
            f.write(f"HOME_AUTO_REFRESH_INTERVAL={self.home_auto_refresh_interval}\n")
            f.write(f"NOTIFICATIONS_AUTO_REFRESH={'on' if self.notifications_auto_refresh else 'off'}\n")
            f.write(f"NOTIFICATIONS_AUTO_REFRESH_INTERVAL={self.notifications_auto_refresh_interval}\n")
            f.write(f"FEDERATED_AUTO_REFRESH={'on' if self.federated_auto_refresh else 'off'}\n")
            f.write(f"FEDERATED_AUTO_REFRESH_INTERVAL={self.federated_auto_refresh_interval}\n")
            f.write(f"IMAGE_SUPPORT={'on' if self.image_support else 'off'}\n")
            f.write(f"IMAGE_RENDERER={self.image_renderer}\n")

    def save_credentials(self, host, client_id, client_secret, access_token):
        self.mastodon_host = host
        self.mastodon_client_id = client_id
        self.mastodon_client_secret = client_secret
        self.mastodon_access_token = access_token
        self.save_config()

config = Config()



from mastodon import Mastodon
from mastui.config import config
from requests import Session


def get_api():
    """Initializes and returns a Mastodon API instance."""
    if config.mastodon_access_token:
        s = Session()
        s.verify = config.ssl_verify
        return Mastodon(
            access_token=config.mastodon_access_token,
            api_base_url=f"https://{config.mastodon_host}",
            session=s,
        )
    return None


def login(host, auth_code):
    """Logs in to a Mastodon instance using an auth code and returns the API object or an error."""
    try:
        s = Session()
        s.verify = config.ssl_verify
        mastodon = Mastodon(
            client_id=config.mastodon_client_id,
            client_secret=config.mastodon_client_secret,
            api_base_url=f"https://{host}",
            
            session=s,
        )
        access_token = mastodon.log_in(
            code=auth_code,
            redirect_uri="urn:ietf:wg:oauth:2.0:oob",
            scopes=["read", "write", "follow", "push"],
        )
        config.save_credentials(
            host, config.mastodon_client_id, config.mastodon_client_secret, access_token
        )
        # Re-initialize with the new access token
        api = Mastodon(
            access_token=access_token,
            api_base_url=f"https://{host}",
        )
        return api, None
    except Exception as e:
        return None, str(e)


def create_app(host):
    """Creates a new Mastodon app and returns the auth URL."""
    try:
        s = Session()
        s.verify = config.ssl_verify
        client_id, client_secret = Mastodon.create_app(
            "mastui",
            api_base_url=f"https://{host}",
            scopes=["read", "write", "follow", "push"],
            redirect_uris="urn:ietf:wg:oauth:2.0:oob",
            session=s,
        )
        # We need to save the client creds to the .env file for the login step
        config.mastodon_host = host
        config.mastodon_client_id = client_id
        config.mastodon_client_secret = client_secret
        config.save_config()

        mastodon = Mastodon(
            client_id=client_id,
            client_secret=client_secret,
            api_base_url=f"https://{host}",
        )
        auth_url = mastodon.auth_request_url(
            redirect_uris="urn:ietf:wg:oauth:2.0:oob",
            scopes=["read", "write", "follow", "push"],
        )
        return auth_url, None
    except Exception as e:
        return None, str(e)



from textual.widgets import Static
import httpx
from io import BytesIO
from textual_image.renderable import Image, SixelImage, HalfcellImage, TGPImage
from PIL import Image as PILImage


class ImageWidget(Static):
    """A widget to display an image."""

    def __init__(self, url: str, renderer: str, **kwargs):
        super().__init__("Loading image...", **kwargs)
        self.url = url
        self.renderer = renderer

    def on_mount(self):
        self.run_worker(self.load_image, thread=True)

    def load_image(self):
        """Loads the image from the URL."""
        try:
            with httpx.stream("GET", self.url, timeout=30) as response:
                response.raise_for_status()
                image_data = response.read()
            img = PILImage.open(BytesIO(image_data))
            self.app.call_from_thread(self.render_image, img)
        except Exception as e:
            self.app.call_from_thread(self.show_error)

    def show_error(self):
        """Displays an error message when the image fails to load."""
        self.update("[Image load failed]")

    def render_image(self, img: PILImage):
        """Renders the image."""
        renderer_map = {
            "auto": Image,
            "sixel": SixelImage,
            "ansi": HalfcellImage,
            "tgp": TGPImage,
        }
        renderer_class = renderer_map.get(self.renderer, Image)
        image = renderer_class(img, width=self.size.width, height="auto")

        # Set the height of the widget to match the image
        self.styles.height = "auto"

        self.update(image)



import pyperclip
from textual.app import App, ComposeResult
from textual.containers import Grid, Vertical
from textual.widgets import Button, Label, Input, Static, TextArea, LoadingIndicator
from textual.screen import ModalScreen

from mastui.mastodon_api import login, create_app

class LoginScreen(ModalScreen):
    """Screen for user to login."""

    DEFAULT_CSS = """
    #dialog {
        grid-size: 2;
        grid-gutter: 1 2;
        grid-rows: auto auto auto 1fr auto;
        padding: 0 1;
        width: 80;
        height: 20;
        border: thick $primary 80%;
        background: $surface;
    }
    """

    def compose(self) -> ComposeResult:
        with Grid(id="dialog"):
            yield Label("Mastodon Instance:")
            yield Input(placeholder="mastodon.social", id="host")
            
            yield Static() # Spacer
            yield Button("Get Auth Link", variant="primary", id="get_auth")

            yield LoadingIndicator(classes="hidden")
            yield Static(id="status", classes="hidden")

            with Vertical(id="auth_link_container", classes="hidden"):
                yield Static("1. Link copied to clipboard! Open it in your browser to authorize.")
                yield TextArea("", id="auth_link",)
                yield Static("2. Paste the authorization code here:")
                yield Input(placeholder="Authorization Code", id="auth_code")
                yield Button("Login", variant="primary", id="login")


    def on_mount(self) -> None:
        """Called when the screen is mounted."""
        self.query_one("#host").focus()

    async def on_button_pressed(self, event: Button.Pressed) -> None:
        status = self.query_one("#status")
        spinner = self.query_one(LoadingIndicator)

        if event.button.id == "get_auth":
            host = self.query_one("#host").value
            if not host:
                status.update("[red]Please enter a Mastodon instance.[/red]")
                status.remove_class("hidden")
                return

            spinner.remove_class("hidden")
            status.add_class("hidden")

            auth_url, error = await self.run_worker(lambda: create_app(host), exclusive=True)
            
            spinner.add_class("hidden")

            if error:
                status.update(f"[red]Error: {error}[/red]")
                status.remove_class("hidden")
                return
            
            pyperclip.set_clipboard("xclip")
            pyperclip.copy(auth_url)

            auth_link_input = self.query_one("#auth_link")
            auth_link_input.text = auth_url
            auth_link_input.read_only = True
            self.query_one("#auth_link_container").remove_class("hidden")
            self.query_one("#get_auth").parent.add_class("hidden") # Hide the button's parent container
            self.query_one("#host").disabled = True
            self.query_one("#auth_code").focus()

        elif event.button.id == "login":
            auth_code = self.query_one("#auth_code").value
            host = self.query_one("#host").value
            if not auth_code:
                status.update("[red]Please enter the authorization code.[/red]")
                status.remove_class("hidden")
                return

            spinner.remove_class("hidden")
            status.add_class("hidden")

            api, error = await self.run_worker(lambda: login(host, auth_code), exclusive=True)

            spinner.add_class("hidden")

            if api:
                self.dismiss(api)
            else:
                status.update(f"[red]Login failed: {error}[/red]")
                status.remove_class("hidden")
